/**
 * DevExtreme (ui/html_editor/modules/toolbar.js)
 * Version: 18.2.0 (build 18288)
 * Build date: Mon Oct 15 2018
 *
 * Copyright (c) 2012 - 2018 Developer Express Inc. ALL RIGHTS RESERVED
 * Read about DevExtreme licensing here: https://js.devexpress.com/Licensing/
 */
"use strict";
var _createClass = function() {
    function defineProperties(target, props) {
        for (var i = 0; i < props.length; i++) {
            var descriptor = props[i];
            descriptor.enumerable = descriptor.enumerable || false;
            descriptor.configurable = true;
            if ("value" in descriptor) {
                descriptor.writable = true
            }
            Object.defineProperty(target, descriptor.key, descriptor)
        }
    }
    return function(Constructor, protoProps, staticProps) {
        if (protoProps) {
            defineProperties(Constructor.prototype, protoProps)
        }
        if (staticProps) {
            defineProperties(Constructor, staticProps)
        }
        return Constructor
    }
}();
var _quill_importer = require("../quill_importer");
var _renderer = require("../../../core/renderer");
var _renderer2 = _interopRequireDefault(_renderer);
var _toolbar = require("../../toolbar");
var _toolbar2 = _interopRequireDefault(_toolbar);
require("../../select_box");
require("../../color_box/color_view");
var _iterator = require("../../../core/utils/iterator");
var _type = require("../../../core/utils/type");
var _extend = require("../../../core/utils/extend");
var _message = require("../../../localization/message");

function _interopRequireDefault(obj) {
    return obj && obj.__esModule ? obj : {
        "default": obj
    }
}

function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
        throw new TypeError("Cannot call a class as a function")
    }
}

function _possibleConstructorReturn(self, call) {
    if (!self) {
        throw new ReferenceError("this hasn't been initialised - super() hasn't been called")
    }
    return call && ("object" === typeof call || "function" === typeof call) ? call : self
}

function _inherits(subClass, superClass) {
    if ("function" !== typeof superClass && null !== superClass) {
        throw new TypeError("Super expression must either be null or a function, not " + typeof superClass)
    }
    subClass.prototype = Object.create(superClass && superClass.prototype, {
        constructor: {
            value: subClass,
            enumerable: false,
            writable: true,
            configurable: true
        }
    });
    if (superClass) {
        Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass
    }
}
var BaseModule = (0, _quill_importer.getQuill)().import("core/module");
var TOOLBAR_CLASS = "dx-htmleditor-toolbar";
var TOOLBAR_FORMAT_WIDGET_CLASS = "dx-htmleditor-toolbar-format";
var ACTIVE_FORMAT_CLASS = "dx-format-active";
var DIALOG_COLOR_CAPTION = "dxHtmlEditor-dialogColorCaption";
var DIALOG_BACKGROUND_CAPTION = "dxHtmlEditor-dialogBackgroundCaption";
var DIALOG_LINK_CAPTION = "dxHtmlEditor-dialogLinkCaption";
var DIALOG_LINK_FIELD_URL = "dxHtmlEditor-dialogLinkUrlField";
var DIALOG_LINK_FIELD_TEXT = "dxHtmlEditor-dialogLinkTextField";
var DIALOG_LINK_FIELD_HINT = "dxHtmlEditor-dialogLinkHintField";
var DIALOG_LINK_FIELD_TARGET = "dxHtmlEditor-dialogLinkTargetField";
var DIALOG_LINK_FIELD_TARGET_CLASS = "dx-formdialog-field-target";
var DIALOG_IMAGE_CAPTION = "dxHtmlEditor-dialogImageCaption";
var DIALOG_IMAGE_FIELD_URL = "dxHtmlEditor-dialogImageUrlField";
var DIALOG_IMAGE_FIELD_ALT = "dxHtmlEditor-dialogImageAltField";
var DIALOG_IMAGE_FIELD_WIDTH = "dxHtmlEditor-dialogImageWidthField";
var DIALOG_IMAGE_FIELD_HEIGHT = "dxHtmlEditor-dialogImageHeightField";
var USER_ACTION = "user";
var ToolbarModule = function(_BaseModule) {
    _inherits(ToolbarModule, _BaseModule);

    function ToolbarModule(quill, options) {
        _classCallCheck(this, ToolbarModule);
        var _this = _possibleConstructorReturn(this, (ToolbarModule.__proto__ || Object.getPrototypeOf(ToolbarModule)).call(this, quill, options));
        _this._editorInstance = options.editorInstance;
        _this._formats = {};
        _this._formatHandlers = _this._getFormatHandlers();
        if ((0, _type.isDefined)(options.items)) {
            _this._renderToolbar();
            _this._editorInstance.on("focusOut", _this._resetFormatWidgets.bind(_this));
            _this.quill.on("editor-change", function(eventName) {
                _this.updateFormatWidgets()
            })
        }
        return _this
    }
    _createClass(ToolbarModule, [{
        key: "_getDefaultClickHandler",
        value: function(formatName) {
            var _this2 = this;
            return function(e) {
                var format = _this2.quill.getFormat(_this2.quill.getSelection());
                var value = !format[formatName];
                _this2.quill.format(formatName, value, USER_ACTION);
                _this2.updateFormatWidgets()
            }
        }
    }, {
        key: "_getFormatHandlers",
        value: function() {
            var _this3 = this;
            return {
                clear: function(e) {
                    _this3.quill.removeFormat(_this3.quill.getSelection());
                    _this3.updateFormatWidgets()
                },
                link: this._prepareLinkHandler(),
                image: this._prepareImageHandler(),
                color: this._prepareColorClickHandler("color"),
                background: this._prepareColorClickHandler("background"),
                orderedList: function() {
                    _this3.quill.format("list", "ordered", USER_ACTION)
                },
                bulletList: function() {
                    _this3.quill.format("list", "bullet", USER_ACTION)
                }
            }
        }
    }, {
        key: "_prepareLinkHandler",
        value: function() {
            var _this4 = this;
            return function() {
                var selection = _this4.quill.getSelection();
                var formats = _this4.quill.getFormat();
                var formData = formats.link || {
                    text: selection ? _this4.quill.getText(selection) : "",
                    target: true
                };
                _this4._editorInstance.formDialogOption("title", (0, _message.format)(DIALOG_LINK_CAPTION));
                var promise = _this4._editorInstance.showFormDialog({
                    formData: formData,
                    items: _this4._getLinkFormItems()
                });
                promise.done(function(formData) {
                    var index = void 0;
                    var length = void 0;
                    if (selection && !formats.link) {
                        index = selection.index;
                        length = selection.length;
                        length && _this4.quill.deleteText(index, length);
                        _this4.quill.setSelection(index, 0, "silent")
                    }
                    _this4.quill.format("link", formData, USER_ACTION)
                })
            }
        }
    }, {
        key: "_getLinkFormItems",
        value: function() {
            return [{
                dataField: "href",
                label: {
                    text: (0, _message.format)(DIALOG_LINK_FIELD_URL)
                }
            }, {
                dataField: "text",
                label: {
                    text: (0, _message.format)(DIALOG_LINK_FIELD_TEXT)
                }
            }, {
                dataField: "title",
                label: {
                    text: (0, _message.format)(DIALOG_LINK_FIELD_HINT)
                }
            }, {
                dataField: "target",
                editorType: "dxCheckBox",
                cssClass: DIALOG_LINK_FIELD_TARGET_CLASS,
                label: {
                    text: (0, _message.format)(DIALOG_LINK_FIELD_TARGET)
                }
            }]
        }
    }, {
        key: "_prepareImageHandler",
        value: function() {
            var _this5 = this;
            return function() {
                var formData = _this5.quill.getFormat();
                var isUpdateDialog = formData.hasOwnProperty("src");
                var selection = _this5.quill.getSelection();
                var pasteIndex = selection && selection.index || _this5.quill.getLength();
                _this5._editorInstance.formDialogOption("title", (0, _message.format)(DIALOG_IMAGE_CAPTION));
                var formItems = [{
                    dataField: "src",
                    label: {
                        text: (0, _message.format)(DIALOG_IMAGE_FIELD_URL)
                    }
                }, {
                    dataField: "width",
                    label: {
                        text: (0, _message.format)(DIALOG_IMAGE_FIELD_WIDTH)
                    }
                }, {
                    dataField: "height",
                    label: {
                        text: (0, _message.format)(DIALOG_IMAGE_FIELD_HEIGHT)
                    }
                }, {
                    dataField: "alt",
                    label: {
                        text: (0, _message.format)(DIALOG_IMAGE_FIELD_ALT)
                    }
                }];
                var promise = _this5._editorInstance.showFormDialog({
                    formData: formData,
                    items: formItems
                });
                promise.done(function(formData) {
                    if (isUpdateDialog) {
                        var formatIndex = selection && !selection.length && selection.index - 1 || pasteIndex;
                        _this5.quill.formatText(formatIndex, 1, {
                            width: formData.width,
                            height: formData.height,
                            alt: formData.alt
                        }, USER_ACTION)
                    } else {
                        _this5.quill.insertEmbed(pasteIndex, "image", formData, USER_ACTION)
                    }
                })
            }
        }
    }, {
        key: "_renderToolbar",
        value: function() {
            var container = this.options.container || this._getContainer();
            var toolbarItems = this._prepareToolbarItems();
            (0, _renderer2.default)(container).addClass(TOOLBAR_CLASS);
            this.toolbarInstance = this._editorInstance._createComponent(container, _toolbar2.default, {
                dataSource: toolbarItems
            })
        }
    }, {
        key: "_getContainer",
        value: function() {
            var $container = (0, _renderer2.default)("<div>");
            this._editorInstance.$element().prepend($container);
            return $container
        }
    }, {
        key: "_prepareToolbarItems",
        value: function() {
            var _this6 = this;
            var resultItems = [];
            (0, _iterator.each)(this.options.items, function(index, item) {
                var newItem = void 0;
                if ((0, _type.isObject)(item)) {
                    if (item.values && !item.widget) {
                        var selectItemConfig = _this6._prepareSelectItemConfig(item);
                        newItem = _this6._getToolbarItem(selectItemConfig)
                    } else {
                        newItem = _this6._getToolbarItem(item)
                    }
                } else {
                    if ((0, _type.isString)(item)) {
                        var buttonItemConfig = _this6._prepareButtonItemConfig(item);
                        newItem = _this6._getToolbarItem(buttonItemConfig)
                    }
                }
                if (newItem) {
                    resultItems.push(newItem)
                }
            });
            return resultItems
        }
    }, {
        key: "_prepareButtonItemConfig",
        value: function(formatName) {
            return {
                widget: "dxButton",
                format: formatName,
                options: {
                    text: formatName,
                    onClick: this._formatHandlers[formatName] || this._getDefaultClickHandler(formatName)
                }
            }
        }
    }, {
        key: "_prepareSelectItemConfig",
        value: function(item) {
            var _this7 = this;
            return {
                widget: "dxSelectBox",
                format: item.format,
                options: {
                    dataSource: item.values,
                    onValueChanged: function(e) {
                        if (!_this7._isReset) {
                            _this7.quill.format(item.format, e.value, USER_ACTION)
                        }
                    }
                }
            }
        }
    }, {
        key: "_prepareColorClickHandler",
        value: function(formatName) {
            var _this8 = this;
            return function() {
                var formData = _this8.quill.getFormat();
                var caption = "color" === formatName ? DIALOG_COLOR_CAPTION : DIALOG_BACKGROUND_CAPTION;
                _this8._editorInstance.formDialogOption("title", (0, _message.format)(caption));
                var promise = _this8._editorInstance.showFormDialog({
                    formData: formData,
                    items: [{
                        dataField: formatName,
                        editorType: "dxColorView",
                        label: {
                            visible: false
                        }
                    }]
                });
                promise.done(function(formData) {
                    _this8.quill.format(formatName, formData[formatName], USER_ACTION)
                })
            }
        }
    }, {
        key: "_getToolbarItem",
        value: function(item) {
            var _this9 = this;
            var baseItem = {
                options: {
                    onInitialized: function(e) {
                        e.component.$element().addClass(TOOLBAR_FORMAT_WIDGET_CLASS);
                        e.component.$element().toggleClass("dx-" + item.format + "-format", !!item.format);
                        _this9._formats[item.format] = e.component
                    }
                }
            };
            return (0, _extend.extend)(true, {
                location: "before"
            }, this._getDefaultConfig(item.format), item, baseItem)
        }
    }, {
        key: "_getDefaultItemsConfig",
        value: function() {
            return {
                header: {
                    options: {
                        displayExpr: function(item) {
                            var isHeaderValue = (0, _type.isDefined)(item) && false !== item;
                            return isHeaderValue ? "H" + item : "Normal"
                        }
                    }
                }
            }
        }
    }, {
        key: "_getDefaultConfig",
        value: function(format) {
            return this._getDefaultItemsConfig()[format]
        }
    }, {
        key: "updateFormatWidgets",
        value: function() {
            var selection = this.quill.getSelection();
            if (!selection) {
                return
            }
            this._resetFormatWidgets();
            var formats = this.quill.getFormat(selection);
            for (var _format in formats) {
                var formatWidget = this._formats[_format];
                if (!formatWidget) {
                    continue
                }
                if (void 0 === formatWidget.option("value")) {
                    formatWidget.$element().addClass(ACTIVE_FORMAT_CLASS)
                } else {
                    this._setValueSilent(formatWidget, formats[_format])
                }
            }
            if (this._formats.clear && !(0, _type.isEmptyObject)(formats)) {
                this._formats.clear.$element().addClass(ACTIVE_FORMAT_CLASS)
            }
        }
    }, {
        key: "_setValueSilent",
        value: function(widget, value) {
            this._isReset = true;
            widget.option("value", value);
            this._isReset = false
        }
    }, {
        key: "_resetFormatWidgets",
        value: function() {
            var _this10 = this;
            (0, _iterator.each)(this._formats, function(name, widget) {
                widget.$element().removeClass(ACTIVE_FORMAT_CLASS);
                if ("dxSelectBox" === widget.NAME) {
                    _this10._setValueSilent(widget, null)
                }
            })
        }
    }, {
        key: "addClickHandler",
        value: function(formatName, handler) {
            this._formatHandlers[formatName] = handler;
            var formatWidget = this._formats[formatName];
            if (formatWidget && "dxButton" === formatWidget.NAME) {
                formatWidget.option("onClick", handler)
            }
        }
    }]);
    return ToolbarModule
}(BaseModule);
module.exports = ToolbarModule;
